# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator (autorest: 3.6.2, generator: @autorest/python@5.12.6)
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class AssetContainerTypeFeedRename(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    WORKSPACE = "Workspace"
    FEED = "Feed"
    REGISTRY = "Registry"


class BatchendpointsUnversionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class DatasetsVersionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class DataStoreAnnotationsDataStoreType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    AZURE_BLOB = "AzureBlob"
    AZURE_FILE = "AzureFile"
    GLUSTER_FS = "GlusterFs"
    AZURE_DATA_LAKE = "AzureDataLake"
    AZURE_MY_SQL = "AzureMySql"
    CUSTOM = "Custom"
    HDFS = "Hdfs"
    AZURE_SQL_DATABASE = "AzureSqlDatabase"
    AZURE_POSTGRE_SQL = "AzurePostgreSql"
    DBFS = "DBFS"
    AZURE_DATA_LAKE_GEN2 = "AzureDataLakeGen2"


class DatastoresUnversionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class EntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class EnvironmentsVersionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class ExperimentStatusCode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    NOT_STARTED = "NotStarted"
    RUNNING = "Running"
    FAILED = "Failed"
    FINISHED = "Finished"
    CANCELED = "Canceled"
    FAILING = "Failing"


class FeatureDataType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    STRING = "String"
    INTEGER = "Integer"
    LONG = "Long"
    FLOAT = "Float"
    DOUBLE = "Double"
    BINARY = "Binary"
    DATETIME = "Datetime"
    BOOLEAN = "Boolean"


class FeatureentitiesVersionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class FeaturesetsVersionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class FeatureSourceDtoType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    MLTABLE = "mltable"
    CSV = "csv"
    PARQUET = "parquet"


class FeaturestoreentitiesVersionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class GenericTriggerAnnotationsEntityStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    ACTIVE = "Active"
    DEPRECATED = "Deprecated"
    DISABLED = "Disabled"


class GenericTriggerAnnotationsManagedType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    CUSTOMER = "Customer"
    SYSTEM = "System"


class GenericTriggerAnnotationsProvisioningStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    COMPLETED = "Completed"
    PROVISIONING = "Provisioning"
    FAILED = "Failed"


class GenericTriggerAnnotationsScheduleActionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    CREATE_JOB = "CreateJob"
    INVOKE_ENDPOINT = "InvokeEndpoint"
    IMPORT_DATA = "ImportData"
    MODEL_MONITOR = "ModelMonitor"


class GenericTriggerAnnotationsScheduleMethod(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    CRON = "Cron"
    RECURRENCE = "Recurrence"


class GenericTriggerAnnotationsTriggerType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    SCHEDULE = "Schedule"
    ONCE = "Once"


class GenerictriggersUnversionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class IndependentPipelineAnnotationsEntityStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    ACTIVE = "Active"
    DEPRECATED = "Deprecated"
    DISABLED = "Disabled"


class IndependentPipelineAnnotationsLastRunStatusCode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    NOT_STARTED = "NotStarted"
    RUNNING = "Running"
    FAILED = "Failed"
    FINISHED = "Finished"
    CANCELED = "Canceled"
    FAILING = "Failing"
    QUEUED = "Queued"
    CANCEL_REQUESTED = "CancelRequested"


class IndependentpipelinesUnversionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class IndexColumnDataType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    STRING = "String"
    INTEGER = "Integer"
    LONG = "Long"
    FLOAT = "Float"
    DOUBLE = "Double"
    BINARY = "Binary"
    DATETIME = "Datetime"
    BOOLEAN = "Boolean"


class IndexColumnDtoType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    STRING = "string"
    INTEGER = "integer"
    LONG = "long"
    FLOAT = "float"
    DOUBLE = "double"
    BINARY = "binary"
    DATETIME = "datetime"
    BOOLEAN = "boolean"


class IndexEntitiesRequestOrderDirection(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    ASC = "Asc"
    DESC = "Desc"


class MaterializationSettingsMaterializationStoreType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    NONE = "None"
    ONLINE = "Online"
    OFFLINE = "Offline"
    ONLINE_AND_OFFLINE = "OnlineAndOffline"


class ModelsVersionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class ModuleAnnotationsAmlModuleEntityStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    ACTIVE = "Active"
    DEPRECATED = "Deprecated"
    DISABLED = "Disabled"


class ModuleAnnotationsEntityStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    ACTIVE = "Active"
    DEPRECATED = "Deprecated"
    DISABLED = "Disabled"


class ModuleAnnotationsModuleType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    USER = "User"
    BUILTIN = "Builtin"


class ModulesVersionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class NotificationSettingEmailOnItem(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    JOB_COMPLETED = "JobCompleted"
    JOB_FAILED = "JobFailed"
    JOB_CANCELLED = "JobCancelled"


class OnlineendpointsUnversionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class PipelineAnnotationsEntityStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    ACTIVE = "Active"
    DEPRECATED = "Deprecated"
    DISABLED = "Disabled"


class PipelineAnnotationsLastRunStatusCode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    NOT_STARTED = "NotStarted"
    RUNNING = "Running"
    FAILED = "Failed"
    FINISHED = "Finished"
    CANCELED = "Canceled"
    FAILING = "Failing"
    QUEUED = "Queued"
    CANCEL_REQUESTED = "CancelRequested"


class PipelineDraftAnnotationsEntityStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    ACTIVE = "Active"
    DEPRECATED = "Deprecated"
    DISABLED = "Disabled"


class PipelinedraftsUnversionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class PipelineRunAnnotationsEntityStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    ACTIVE = "Active"
    DEPRECATED = "Deprecated"
    DISABLED = "Disabled"


class PipelinerunsUnversionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class PipelinesVersionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class RecurrenceFrequency(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    MONTH = "Month"
    WEEK = "Week"
    DAY = "Day"
    HOUR = "Hour"
    MINUTE = "Minute"


class RecurrenceScheduleWeekDaysItem(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    MONDAY = "Monday"
    TUESDAY = "Tuesday"
    WEDNESDAY = "Wednesday"
    THURSDAY = "Thursday"
    FRIDAY = "Friday"
    SATURDAY = "Saturday"
    SUNDAY = "Sunday"


class RecurrenceTriggerFrequency(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    MONTH = "Month"
    WEEK = "Week"
    DAY = "Day"
    HOUR = "Hour"
    MINUTE = "Minute"


class RunsUnversionedEntityKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INVALID = "Invalid"
    LINEAGE_ROOT = "LineageRoot"
    VERSIONED = "Versioned"
    UNVERSIONED = "Unversioned"


class StructuredInterfaceParameterType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    INT = "Int"
    DOUBLE = "Double"
    BOOL = "Bool"
    STRING = "String"
    UNDEFINED = "Undefined"
