# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._azure_machine_learning_services_enums import (
    ActionType,
    AllocationState,
    ApplicationSharingPolicy,
    AssetProvisioningState,
    AutoRebuildSetting,
    Autosave,
    BatchLoggingLevel,
    BatchOutputAction,
    BillingCurrency,
    BlockedTransformers,
    Caching,
    CategoricalDataDriftMetric,
    CategoricalDataQualityMetric,
    CategoricalPredictionDriftMetric,
    ClassificationModels,
    ClassificationMultilabelPrimaryMetrics,
    ClassificationPrimaryMetrics,
    ClusterPurpose,
    ComputeInstanceAuthorizationType,
    ComputeInstanceState,
    ComputePowerAction,
    ComputeRecurrenceFrequency,
    ComputeTriggerType,
    ComputeType,
    ComputeWeekDay,
    ConnectionAuthType,
    ConnectionCategory,
    ContainerType,
    CreatedByType,
    CredentialsType,
    DataAvailabilityStatus,
    DataReferenceCredentialType,
    DatastoreType,
    DataType,
    DeploymentProvisioningState,
    DiagnoseResultLevel,
    DistributionType,
    EarlyTerminationPolicyType,
    EgressPublicNetworkAccessType,
    EmailNotificationEnableType,
    EncryptionStatus,
    EndpointAuthMode,
    EndpointComputeType,
    EndpointProvisioningState,
    EndpointServiceConnectionStatus,
    EnvironmentType,
    EnvironmentVariableType,
    FeatureAttributionMetric,
    FeatureDataType,
    FeatureImportanceMode,
    FeatureLags,
    FeaturizationMode,
    ForecastHorizonMode,
    ForecastingModels,
    ForecastingPrimaryMetrics,
    Goal,
    IdentityConfigurationType,
    ImageType,
    InputDeliveryMode,
    InstanceSegmentationPrimaryMetrics,
    IsolationMode,
    JobInputType,
    JobLimitsType,
    JobOutputType,
    JobStatus,
    JobTier,
    JobType,
    KeyType,
    LearningRateScheduler,
    ListViewType,
    LoadBalancerType,
    LogVerbosity,
    ManagedNetworkStatus,
    ManagedServiceIdentityType,
    MaterializationStoreType,
    ModelSize,
    ModelTaskType,
    MonitorComputeIdentityType,
    MonitorComputeType,
    MonitoringFeatureDataType,
    MonitoringFeatureFilterType,
    MonitoringInputDataType,
    MonitoringNotificationType,
    MonitoringSignalType,
    MountAction,
    MountState,
    NCrossValidationsMode,
    Network,
    NodeState,
    NodesValueType,
    NumericalDataDriftMetric,
    NumericalDataQualityMetric,
    NumericalPredictionDriftMetric,
    ObjectDetectionPrimaryMetrics,
    OperatingSystemType,
    OperationName,
    OperationStatus,
    OperationTrigger,
    OrderString,
    Origin,
    OsType,
    OutputDeliveryMode,
    PendingUploadCredentialType,
    PendingUploadType,
    PrivateEndpointConnectionProvisioningState,
    PrivateEndpointServiceConnectionStatus,
    Protocol,
    ProvisioningState,
    ProvisioningStatus,
    PublicNetworkAccess,
    PublicNetworkAccessType,
    QuotaUnit,
    RandomSamplingAlgorithmRule,
    RecurrenceFrequency,
    ReferenceType,
    RegressionModels,
    RegressionPrimaryMetrics,
    RemoteLoginPortPublicAccess,
    RuleAction,
    RuleCategory,
    RuleStatus,
    RuleType,
    SamplingAlgorithmType,
    ScaleType,
    ScheduleActionType,
    ScheduleListViewType,
    ScheduleProvisioningState,
    ScheduleProvisioningStatus,
    ScheduleStatus,
    SeasonalityMode,
    SecretsType,
    ServiceDataAccessAuthIdentity,
    ShortSeriesHandlingConfiguration,
    SkuScaleType,
    SkuTier,
    SourceType,
    SshPublicAccess,
    SslConfigStatus,
    StackMetaLearnerType,
    Status,
    StochasticOptimizer,
    StorageAccountType,
    TargetAggregationFunction,
    TargetLagsMode,
    TargetRollingWindowSizeMode,
    TaskType,
    TriggerType,
    UnderlyingResourceAction,
    UnitOfMeasure,
    UsageUnit,
    UseStl,
    ValidationMetricType,
    ValueFormat,
    VMPriceOSType,
    VmPriority,
    VMTier,
    VolumeDefinitionType,
    WebhookType,
    WeekDay,
)
from ._models_py3 import (
    AKS,
    AccountKeyDatastoreCredentials,
    AccountKeyDatastoreSecrets,
    AcrDetails,
    AksComputeSecrets,
    AksComputeSecretsProperties,
    AksNetworkingConfiguration,
    AKSSchema,
    AKSSchemaProperties,
    AllFeatures,
    AllNodes,
    AmlCompute,
    AmlComputeNodeInformation,
    AmlComputeNodesInformation,
    AmlComputeProperties,
    AmlComputeSchema,
    AmlToken,
    AmlTokenComputeIdentity,
    AmlUserFeature,
    AnonymousAccessCredential,
    ArmResourceId,
    AssetBase,
    AssetContainer,
    AssetJobInput,
    AssetJobOutput,
    AssetReferenceBase,
    AssignedUser,
    AutoForecastHorizon,
    AutoMLJob,
    AutoMLVertical,
    AutoNCrossValidations,
    AutoPauseProperties,
    AutoScaleProperties,
    AutoSeasonality,
    AutoTargetLags,
    AutoTargetRollingWindowSize,
    AzureBlobDatastore,
    AzureDataLakeGen1Datastore,
    AzureDataLakeGen2Datastore,
    AzureDatastore,
    AzureDevOpsWebhook,
    AzureFileDatastore,
    BanditPolicy,
    BatchDeployment,
    BatchDeploymentProperties,
    BatchDeploymentTrackedResourceArmPaginatedResult,
    BatchEndpoint,
    BatchEndpointDefaults,
    BatchEndpointProperties,
    BatchEndpointTrackedResourceArmPaginatedResult,
    BatchRetrySettings,
    BayesianSamplingAlgorithm,
    BindOptions,
    BlobReferenceForConsumptionDto,
    BuildContext,
    CategoricalDataDriftMetricThreshold,
    CategoricalDataQualityMetricThreshold,
    CategoricalPredictionDriftMetricThreshold,
    CertificateDatastoreCredentials,
    CertificateDatastoreSecrets,
    Classification,
    ClassificationTrainingSettings,
    ClusterUpdateParameters,
    CodeConfiguration,
    CodeContainer,
    CodeContainerProperties,
    CodeContainerResourceArmPaginatedResult,
    CodeVersion,
    CodeVersionProperties,
    CodeVersionResourceArmPaginatedResult,
    ColumnTransformer,
    CommandJob,
    CommandJobLimits,
    ComponentContainer,
    ComponentContainerProperties,
    ComponentContainerResourceArmPaginatedResult,
    ComponentVersion,
    ComponentVersionProperties,
    ComponentVersionResourceArmPaginatedResult,
    Compute,
    ComputeInstance,
    ComputeInstanceApplication,
    ComputeInstanceConnectivityEndpoints,
    ComputeInstanceContainer,
    ComputeInstanceCreatedBy,
    ComputeInstanceDataDisk,
    ComputeInstanceDataMount,
    ComputeInstanceEnvironmentInfo,
    ComputeInstanceLastOperation,
    ComputeInstanceProperties,
    ComputeInstanceSchema,
    ComputeInstanceSshSettings,
    ComputeInstanceVersion,
    ComputeRecurrenceSchedule,
    ComputeResource,
    ComputeResourceSchema,
    ComputeSchedules,
    ComputeSecrets,
    ComputeStartStopSchedule,
    ContainerResourceRequirements,
    ContainerResourceSettings,
    CosmosDbSettings,
    CreateMonitorAction,
    Cron,
    CronTrigger,
    CustomForecastHorizon,
    CustomMetricThreshold,
    CustomModelJobInput,
    CustomModelJobOutput,
    CustomMonitoringSignal,
    CustomNCrossValidations,
    CustomSeasonality,
    CustomService,
    CustomTargetLags,
    CustomTargetRollingWindowSize,
    Databricks,
    DatabricksComputeSecrets,
    DatabricksComputeSecretsProperties,
    DatabricksProperties,
    DatabricksSchema,
    DataContainer,
    DataContainerProperties,
    DataContainerResourceArmPaginatedResult,
    DataDriftMetricThresholdBase,
    DataDriftMonitoringSignal,
    DataFactory,
    DataLakeAnalytics,
    DataLakeAnalyticsSchema,
    DataLakeAnalyticsSchemaProperties,
    DataPathAssetReference,
    DataQualityMetricThresholdBase,
    DataQualityMonitoringSignal,
    DataReferenceCredential,
    Datastore,
    DatastoreCredentials,
    DatastoreProperties,
    DatastoreResourceArmPaginatedResult,
    DatastoreSecrets,
    DataVersionBase,
    DataVersionBaseProperties,
    DataVersionBaseResourceArmPaginatedResult,
    DefaultScaleSettings,
    DeploymentLogs,
    DeploymentLogsRequest,
    DeploymentResourceConfiguration,
    DestinationAsset,
    DiagnoseRequestProperties,
    DiagnoseResponseResult,
    DiagnoseResponseResultValue,
    DiagnoseResult,
    DiagnoseWorkspaceParameters,
    DistributionConfiguration,
    Docker,
    DockerCredential,
    EarlyTerminationPolicy,
    EncryptionKeyVaultProperties,
    EncryptionProperty,
    Endpoint,
    EndpointAuthKeys,
    EndpointAuthToken,
    EndpointDeploymentPropertiesBase,
    EndpointPropertiesBase,
    EndpointScheduleAction,
    EnvironmentContainer,
    EnvironmentContainerProperties,
    EnvironmentContainerResourceArmPaginatedResult,
    EnvironmentVariable,
    EnvironmentVersion,
    EnvironmentVersionProperties,
    EnvironmentVersionResourceArmPaginatedResult,
    ErrorAdditionalInfo,
    ErrorDetail,
    ErrorResponse,
    EstimatedVMPrice,
    EstimatedVMPrices,
    ExternalFQDNResponse,
    Feature,
    FeatureAttributionDriftMonitoringSignal,
    FeatureAttributionMetricThreshold,
    FeatureImportanceSettings,
    FeatureProperties,
    FeatureResourceArmPaginatedResult,
    FeaturesetContainer,
    FeaturesetContainerProperties,
    FeaturesetContainerResourceArmPaginatedResult,
    FeaturesetSpecification,
    FeaturesetVersion,
    FeaturesetVersionBackfillRequest,
    FeaturesetVersionBackfillResponse,
    FeaturesetVersionProperties,
    FeaturesetVersionResourceArmPaginatedResult,
    FeaturestoreEntityContainer,
    FeaturestoreEntityContainerProperties,
    FeaturestoreEntityContainerResourceArmPaginatedResult,
    FeaturestoreEntityVersion,
    FeaturestoreEntityVersionProperties,
    FeaturestoreEntityVersionResourceArmPaginatedResult,
    FeatureSubset,
    FeatureWindow,
    FeaturizationSettings,
    FixedInputData,
    FlavorData,
    ForecastHorizon,
    Forecasting,
    ForecastingSettings,
    ForecastingTrainingSettings,
    FQDNEndpoint,
    FQDNEndpointDetail,
    FQDNEndpoints,
    FQDNEndpointsProperties,
    FqdnOutboundRule,
    GetBlobReferenceForConsumptionDto,
    GetBlobReferenceSASRequestDto,
    GetBlobReferenceSASResponseDto,
    GridSamplingAlgorithm,
    HDInsight,
    HDInsightProperties,
    HDInsightSchema,
    IdAssetReference,
    IdentityConfiguration,
    IdentityForCmk,
    IdleShutdownSetting,
    Image,
    ImageClassification,
    ImageClassificationBase,
    ImageClassificationMultilabel,
    ImageInstanceSegmentation,
    ImageLimitSettings,
    ImageMetadata,
    ImageModelDistributionSettings,
    ImageModelDistributionSettingsClassification,
    ImageModelDistributionSettingsObjectDetection,
    ImageModelSettings,
    ImageModelSettingsClassification,
    ImageModelSettingsObjectDetection,
    ImageObjectDetection,
    ImageObjectDetectionBase,
    ImageSweepSettings,
    ImageVertical,
    IndexColumn,
    InferenceContainerProperties,
    InstanceTypeSchema,
    InstanceTypeSchemaResources,
    JobBase,
    JobBaseProperties,
    JobBaseResourceArmPaginatedResult,
    JobInput,
    JobLimits,
    JobOutput,
    JobResourceConfiguration,
    JobScheduleAction,
    JobService,
    Kubernetes,
    KubernetesOnlineDeployment,
    KubernetesProperties,
    KubernetesSchema,
    ListAmlUserFeatureResult,
    ListNotebookKeysResult,
    ListStorageAccountKeysResult,
    ListUsagesResult,
    ListWorkspaceKeysResult,
    ListWorkspaceQuotas,
    LiteralJobInput,
    ManagedComputeIdentity,
    ManagedIdentity,
    ManagedIdentityAuthTypeWorkspaceConnectionProperties,
    ManagedIdentityCredential,
    ManagedNetworkProvisionOptions,
    ManagedNetworkProvisionStatus,
    ManagedNetworkSettings,
    ManagedOnlineDeployment,
    ManagedServiceIdentity,
    MaterializationComputeResource,
    MaterializationSettings,
    MedianStoppingPolicy,
    MLFlowModelJobInput,
    MLFlowModelJobOutput,
    MLTableData,
    MLTableJobInput,
    MLTableJobOutput,
    ModelContainer,
    ModelContainerProperties,
    ModelContainerResourceArmPaginatedResult,
    ModelVersion,
    ModelVersionProperties,
    ModelVersionResourceArmPaginatedResult,
    MonitorComputeConfigurationBase,
    MonitorComputeIdentityBase,
    MonitorDefinition,
    MonitorEmailNotificationSettings,
    MonitoringFeatureFilterBase,
    MonitoringInputDataBase,
    MonitoringSignalBase,
    MonitoringTarget,
    MonitoringThreshold,
    MonitorNotificationSettings,
    MonitorServerlessSparkCompute,
    Mpi,
    NCrossValidations,
    NlpVertical,
    NlpVerticalFeaturizationSettings,
    NlpVerticalLimitSettings,
    Nodes,
    NodeStateCounts,
    NoneAuthTypeWorkspaceConnectionProperties,
    NoneDatastoreCredentials,
    NotebookAccessTokenResult,
    NotebookPreparationError,
    NotebookResourceInfo,
    NotificationSetting,
    NumericalDataDriftMetricThreshold,
    NumericalDataQualityMetricThreshold,
    NumericalPredictionDriftMetricThreshold,
    Objective,
    OnlineDeployment,
    OnlineDeploymentProperties,
    OnlineDeploymentTrackedResourceArmPaginatedResult,
    OnlineEndpoint,
    OnlineEndpointProperties,
    OnlineEndpointTrackedResourceArmPaginatedResult,
    OnlineRequestSettings,
    OnlineScaleSettings,
    Operation,
    OperationDisplay,
    OperationListResult,
    OutboundRule,
    OutboundRuleBasicResource,
    OutboundRuleListResult,
    OutputPathAssetReference,
    PaginatedComputeResourcesList,
    PartialBatchDeployment,
    PartialBatchDeploymentPartialMinimalTrackedResourceWithProperties,
    PartialManagedServiceIdentity,
    PartialMinimalTrackedResource,
    PartialMinimalTrackedResourceWithIdentity,
    PartialMinimalTrackedResourceWithSku,
    PartialRegistryPartialTrackedResource,
    PartialSku,
    Password,
    PATAuthTypeWorkspaceConnectionProperties,
    PendingUploadCredentialDto,
    PendingUploadRequestDto,
    PendingUploadResponseDto,
    PersonalComputeInstanceSettings,
    PipelineJob,
    PredictionDriftMetricThresholdBase,
    PredictionDriftMonitoringSignal,
    PrivateEndpoint,
    PrivateEndpointConnection,
    PrivateEndpointConnectionListResult,
    PrivateEndpointDestination,
    PrivateEndpointOutboundRule,
    PrivateEndpointResource,
    PrivateLinkResource,
    PrivateLinkResourceListResult,
    PrivateLinkServiceConnectionState,
    ProbeSettings,
    ProxyResource,
    PyTorch,
    QueueSettings,
    QuotaBaseProperties,
    QuotaUpdateParameters,
    RandomSamplingAlgorithm,
    Recurrence,
    RecurrenceSchedule,
    RecurrenceTrigger,
    RegenerateEndpointKeysRequest,
    Registry,
    RegistryListCredentialsResult,
    RegistryPartialManagedServiceIdentity,
    RegistryPrivateEndpointConnection,
    RegistryPrivateLinkServiceConnectionState,
    RegistryRegionArmDetails,
    RegistryTrackedResourceArmPaginatedResult,
    Regression,
    RegressionTrainingSettings,
    Resource,
    ResourceBase,
    ResourceConfiguration,
    ResourceId,
    ResourceName,
    ResourceQuota,
    RollingInputData,
    Route,
    SamplingAlgorithm,
    SASAuthTypeWorkspaceConnectionProperties,
    SASCredential,
    SASCredentialDto,
    SasDatastoreCredentials,
    SasDatastoreSecrets,
    ScaleSettings,
    ScaleSettingsInformation,
    Schedule,
    ScheduleActionBase,
    ScheduleBase,
    ScheduleProperties,
    ScheduleResourceArmPaginatedResult,
    ScriptReference,
    ScriptsToExecute,
    Seasonality,
    ServerlessComputeSettings,
    ServiceManagedResourcesSettings,
    ServicePrincipalDatastoreCredentials,
    ServicePrincipalDatastoreSecrets,
    ServiceTagDestination,
    ServiceTagOutboundRule,
    SetupScripts,
    SharedPrivateLinkResource,
    Sku,
    SkuCapacity,
    SkuResource,
    SkuResourceArmPaginatedResult,
    SkuSetting,
    SslConfiguration,
    StackEnsembleSettings,
    StaticInputData,
    StorageAccountDetails,
    SweepJob,
    SweepJobLimits,
    SynapseSpark,
    SynapseSparkProperties,
    SystemCreatedAcrAccount,
    SystemCreatedStorageAccount,
    SystemData,
    SystemService,
    TableVertical,
    TableVerticalFeaturizationSettings,
    TableVerticalLimitSettings,
    TargetLags,
    TargetRollingWindowSize,
    TargetUtilizationScaleSettings,
    TensorFlow,
    TextClassification,
    TextClassificationMultilabel,
    TextNer,
    TmpfsOptions,
    TopNFeaturesByAttribution,
    TrackedResource,
    TrainingSettings,
    TrialComponent,
    TriggerBase,
    TritonModelJobInput,
    TritonModelJobOutput,
    TruncationSelectionPolicy,
    UpdateWorkspaceQuotas,
    UpdateWorkspaceQuotasResult,
    UriFileDataVersion,
    UriFileJobInput,
    UriFileJobOutput,
    UriFolderDataVersion,
    UriFolderJobInput,
    UriFolderJobOutput,
    Usage,
    UsageName,
    UserAccountCredentials,
    UserAssignedIdentity,
    UserCreatedAcrAccount,
    UserCreatedStorageAccount,
    UserIdentity,
    UsernamePasswordAuthTypeWorkspaceConnectionProperties,
    VirtualMachine,
    VirtualMachineImage,
    VirtualMachineSchema,
    VirtualMachineSchemaProperties,
    VirtualMachineSecrets,
    VirtualMachineSecretsSchema,
    VirtualMachineSize,
    VirtualMachineSizeListResult,
    VirtualMachineSshCredentials,
    VolumeDefinition,
    VolumeOptions,
    Webhook,
    Workspace,
    WorkspaceConnectionManagedIdentity,
    WorkspaceConnectionPersonalAccessToken,
    WorkspaceConnectionPropertiesV2,
    WorkspaceConnectionPropertiesV2BasicResource,
    WorkspaceConnectionPropertiesV2BasicResourceArmPaginatedResult,
    WorkspaceConnectionSharedAccessSignature,
    WorkspaceConnectionUsernamePassword,
    WorkspaceListResult,
    WorkspaceUpdateParameters,
)
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import __all__ as _patch_all
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AKS",
    "AKSSchema",
    "AKSSchemaProperties",
    "AccountKeyDatastoreCredentials",
    "AccountKeyDatastoreSecrets",
    "AcrDetails",
    "AksComputeSecrets",
    "AksComputeSecretsProperties",
    "AksNetworkingConfiguration",
    "AllFeatures",
    "AllNodes",
    "AmlCompute",
    "AmlComputeNodeInformation",
    "AmlComputeNodesInformation",
    "AmlComputeProperties",
    "AmlComputeSchema",
    "AmlToken",
    "AmlTokenComputeIdentity",
    "AmlUserFeature",
    "AnonymousAccessCredential",
    "ArmResourceId",
    "AssetBase",
    "AssetContainer",
    "AssetJobInput",
    "AssetJobOutput",
    "AssetReferenceBase",
    "AssignedUser",
    "AutoForecastHorizon",
    "AutoMLJob",
    "AutoMLVertical",
    "AutoNCrossValidations",
    "AutoPauseProperties",
    "AutoScaleProperties",
    "AutoSeasonality",
    "AutoTargetLags",
    "AutoTargetRollingWindowSize",
    "AzureBlobDatastore",
    "AzureDataLakeGen1Datastore",
    "AzureDataLakeGen2Datastore",
    "AzureDatastore",
    "AzureDevOpsWebhook",
    "AzureFileDatastore",
    "BanditPolicy",
    "BatchDeployment",
    "BatchDeploymentProperties",
    "BatchDeploymentTrackedResourceArmPaginatedResult",
    "BatchEndpoint",
    "BatchEndpointDefaults",
    "BatchEndpointProperties",
    "BatchEndpointTrackedResourceArmPaginatedResult",
    "BatchRetrySettings",
    "BayesianSamplingAlgorithm",
    "BindOptions",
    "BlobReferenceForConsumptionDto",
    "BuildContext",
    "CategoricalDataDriftMetricThreshold",
    "CategoricalDataQualityMetricThreshold",
    "CategoricalPredictionDriftMetricThreshold",
    "CertificateDatastoreCredentials",
    "CertificateDatastoreSecrets",
    "Classification",
    "ClassificationTrainingSettings",
    "ClusterUpdateParameters",
    "CodeConfiguration",
    "CodeContainer",
    "CodeContainerProperties",
    "CodeContainerResourceArmPaginatedResult",
    "CodeVersion",
    "CodeVersionProperties",
    "CodeVersionResourceArmPaginatedResult",
    "ColumnTransformer",
    "CommandJob",
    "CommandJobLimits",
    "ComponentContainer",
    "ComponentContainerProperties",
    "ComponentContainerResourceArmPaginatedResult",
    "ComponentVersion",
    "ComponentVersionProperties",
    "ComponentVersionResourceArmPaginatedResult",
    "Compute",
    "ComputeInstance",
    "ComputeInstanceApplication",
    "ComputeInstanceConnectivityEndpoints",
    "ComputeInstanceContainer",
    "ComputeInstanceCreatedBy",
    "ComputeInstanceDataDisk",
    "ComputeInstanceDataMount",
    "ComputeInstanceEnvironmentInfo",
    "ComputeInstanceLastOperation",
    "ComputeInstanceProperties",
    "ComputeInstanceSchema",
    "ComputeInstanceSshSettings",
    "ComputeInstanceVersion",
    "ComputeRecurrenceSchedule",
    "ComputeResource",
    "ComputeResourceSchema",
    "ComputeSchedules",
    "ComputeSecrets",
    "ComputeStartStopSchedule",
    "ContainerResourceRequirements",
    "ContainerResourceSettings",
    "CosmosDbSettings",
    "CreateMonitorAction",
    "Cron",
    "CronTrigger",
    "CustomForecastHorizon",
    "CustomMetricThreshold",
    "CustomModelJobInput",
    "CustomModelJobOutput",
    "CustomMonitoringSignal",
    "CustomNCrossValidations",
    "CustomSeasonality",
    "CustomService",
    "CustomTargetLags",
    "CustomTargetRollingWindowSize",
    "DataContainer",
    "DataContainerProperties",
    "DataContainerResourceArmPaginatedResult",
    "DataDriftMetricThresholdBase",
    "DataDriftMonitoringSignal",
    "DataFactory",
    "DataLakeAnalytics",
    "DataLakeAnalyticsSchema",
    "DataLakeAnalyticsSchemaProperties",
    "DataPathAssetReference",
    "DataQualityMetricThresholdBase",
    "DataQualityMonitoringSignal",
    "DataReferenceCredential",
    "DataVersionBase",
    "DataVersionBaseProperties",
    "DataVersionBaseResourceArmPaginatedResult",
    "Databricks",
    "DatabricksComputeSecrets",
    "DatabricksComputeSecretsProperties",
    "DatabricksProperties",
    "DatabricksSchema",
    "Datastore",
    "DatastoreCredentials",
    "DatastoreProperties",
    "DatastoreResourceArmPaginatedResult",
    "DatastoreSecrets",
    "DefaultScaleSettings",
    "DeploymentLogs",
    "DeploymentLogsRequest",
    "DeploymentResourceConfiguration",
    "DestinationAsset",
    "DiagnoseRequestProperties",
    "DiagnoseResponseResult",
    "DiagnoseResponseResultValue",
    "DiagnoseResult",
    "DiagnoseWorkspaceParameters",
    "DistributionConfiguration",
    "Docker",
    "DockerCredential",
    "EarlyTerminationPolicy",
    "EncryptionKeyVaultProperties",
    "EncryptionProperty",
    "Endpoint",
    "EndpointAuthKeys",
    "EndpointAuthToken",
    "EndpointDeploymentPropertiesBase",
    "EndpointPropertiesBase",
    "EndpointScheduleAction",
    "EnvironmentContainer",
    "EnvironmentContainerProperties",
    "EnvironmentContainerResourceArmPaginatedResult",
    "EnvironmentVariable",
    "EnvironmentVersion",
    "EnvironmentVersionProperties",
    "EnvironmentVersionResourceArmPaginatedResult",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ErrorResponse",
    "EstimatedVMPrice",
    "EstimatedVMPrices",
    "ExternalFQDNResponse",
    "FQDNEndpoint",
    "FQDNEndpointDetail",
    "FQDNEndpoints",
    "FQDNEndpointsProperties",
    "Feature",
    "FeatureAttributionDriftMonitoringSignal",
    "FeatureAttributionMetricThreshold",
    "FeatureImportanceSettings",
    "FeatureProperties",
    "FeatureResourceArmPaginatedResult",
    "FeatureSubset",
    "FeatureWindow",
    "FeaturesetContainer",
    "FeaturesetContainerProperties",
    "FeaturesetContainerResourceArmPaginatedResult",
    "FeaturesetSpecification",
    "FeaturesetVersion",
    "FeaturesetVersionBackfillRequest",
    "FeaturesetVersionBackfillResponse",
    "FeaturesetVersionProperties",
    "FeaturesetVersionResourceArmPaginatedResult",
    "FeaturestoreEntityContainer",
    "FeaturestoreEntityContainerProperties",
    "FeaturestoreEntityContainerResourceArmPaginatedResult",
    "FeaturestoreEntityVersion",
    "FeaturestoreEntityVersionProperties",
    "FeaturestoreEntityVersionResourceArmPaginatedResult",
    "FeaturizationSettings",
    "FixedInputData",
    "FlavorData",
    "ForecastHorizon",
    "Forecasting",
    "ForecastingSettings",
    "ForecastingTrainingSettings",
    "FqdnOutboundRule",
    "GetBlobReferenceForConsumptionDto",
    "GetBlobReferenceSASRequestDto",
    "GetBlobReferenceSASResponseDto",
    "GridSamplingAlgorithm",
    "HDInsight",
    "HDInsightProperties",
    "HDInsightSchema",
    "IdAssetReference",
    "IdentityConfiguration",
    "IdentityForCmk",
    "IdleShutdownSetting",
    "Image",
    "ImageClassification",
    "ImageClassificationBase",
    "ImageClassificationMultilabel",
    "ImageInstanceSegmentation",
    "ImageLimitSettings",
    "ImageMetadata",
    "ImageModelDistributionSettings",
    "ImageModelDistributionSettingsClassification",
    "ImageModelDistributionSettingsObjectDetection",
    "ImageModelSettings",
    "ImageModelSettingsClassification",
    "ImageModelSettingsObjectDetection",
    "ImageObjectDetection",
    "ImageObjectDetectionBase",
    "ImageSweepSettings",
    "ImageVertical",
    "IndexColumn",
    "InferenceContainerProperties",
    "InstanceTypeSchema",
    "InstanceTypeSchemaResources",
    "JobBase",
    "JobBaseProperties",
    "JobBaseResourceArmPaginatedResult",
    "JobInput",
    "JobLimits",
    "JobOutput",
    "JobResourceConfiguration",
    "JobScheduleAction",
    "JobService",
    "Kubernetes",
    "KubernetesOnlineDeployment",
    "KubernetesProperties",
    "KubernetesSchema",
    "ListAmlUserFeatureResult",
    "ListNotebookKeysResult",
    "ListStorageAccountKeysResult",
    "ListUsagesResult",
    "ListWorkspaceKeysResult",
    "ListWorkspaceQuotas",
    "LiteralJobInput",
    "MLFlowModelJobInput",
    "MLFlowModelJobOutput",
    "MLTableData",
    "MLTableJobInput",
    "MLTableJobOutput",
    "ManagedComputeIdentity",
    "ManagedIdentity",
    "ManagedIdentityAuthTypeWorkspaceConnectionProperties",
    "ManagedIdentityCredential",
    "ManagedNetworkProvisionOptions",
    "ManagedNetworkProvisionStatus",
    "ManagedNetworkSettings",
    "ManagedOnlineDeployment",
    "ManagedServiceIdentity",
    "MaterializationComputeResource",
    "MaterializationSettings",
    "MedianStoppingPolicy",
    "ModelContainer",
    "ModelContainerProperties",
    "ModelContainerResourceArmPaginatedResult",
    "ModelVersion",
    "ModelVersionProperties",
    "ModelVersionResourceArmPaginatedResult",
    "MonitorComputeConfigurationBase",
    "MonitorComputeIdentityBase",
    "MonitorDefinition",
    "MonitorEmailNotificationSettings",
    "MonitorNotificationSettings",
    "MonitorServerlessSparkCompute",
    "MonitoringFeatureFilterBase",
    "MonitoringInputDataBase",
    "MonitoringSignalBase",
    "MonitoringTarget",
    "MonitoringThreshold",
    "Mpi",
    "NCrossValidations",
    "NlpVertical",
    "NlpVerticalFeaturizationSettings",
    "NlpVerticalLimitSettings",
    "NodeStateCounts",
    "Nodes",
    "NoneAuthTypeWorkspaceConnectionProperties",
    "NoneDatastoreCredentials",
    "NotebookAccessTokenResult",
    "NotebookPreparationError",
    "NotebookResourceInfo",
    "NotificationSetting",
    "NumericalDataDriftMetricThreshold",
    "NumericalDataQualityMetricThreshold",
    "NumericalPredictionDriftMetricThreshold",
    "Objective",
    "OnlineDeployment",
    "OnlineDeploymentProperties",
    "OnlineDeploymentTrackedResourceArmPaginatedResult",
    "OnlineEndpoint",
    "OnlineEndpointProperties",
    "OnlineEndpointTrackedResourceArmPaginatedResult",
    "OnlineRequestSettings",
    "OnlineScaleSettings",
    "Operation",
    "OperationDisplay",
    "OperationListResult",
    "OutboundRule",
    "OutboundRuleBasicResource",
    "OutboundRuleListResult",
    "OutputPathAssetReference",
    "PATAuthTypeWorkspaceConnectionProperties",
    "PaginatedComputeResourcesList",
    "PartialBatchDeployment",
    "PartialBatchDeploymentPartialMinimalTrackedResourceWithProperties",
    "PartialManagedServiceIdentity",
    "PartialMinimalTrackedResource",
    "PartialMinimalTrackedResourceWithIdentity",
    "PartialMinimalTrackedResourceWithSku",
    "PartialRegistryPartialTrackedResource",
    "PartialSku",
    "Password",
    "PendingUploadCredentialDto",
    "PendingUploadRequestDto",
    "PendingUploadResponseDto",
    "PersonalComputeInstanceSettings",
    "PipelineJob",
    "PredictionDriftMetricThresholdBase",
    "PredictionDriftMonitoringSignal",
    "PrivateEndpoint",
    "PrivateEndpointConnection",
    "PrivateEndpointConnectionListResult",
    "PrivateEndpointDestination",
    "PrivateEndpointOutboundRule",
    "PrivateEndpointResource",
    "PrivateLinkResource",
    "PrivateLinkResourceListResult",
    "PrivateLinkServiceConnectionState",
    "ProbeSettings",
    "ProxyResource",
    "PyTorch",
    "QueueSettings",
    "QuotaBaseProperties",
    "QuotaUpdateParameters",
    "RandomSamplingAlgorithm",
    "Recurrence",
    "RecurrenceSchedule",
    "RecurrenceTrigger",
    "RegenerateEndpointKeysRequest",
    "Registry",
    "RegistryListCredentialsResult",
    "RegistryPartialManagedServiceIdentity",
    "RegistryPrivateEndpointConnection",
    "RegistryPrivateLinkServiceConnectionState",
    "RegistryRegionArmDetails",
    "RegistryTrackedResourceArmPaginatedResult",
    "Regression",
    "RegressionTrainingSettings",
    "Resource",
    "ResourceBase",
    "ResourceConfiguration",
    "ResourceId",
    "ResourceName",
    "ResourceQuota",
    "RollingInputData",
    "Route",
    "SASAuthTypeWorkspaceConnectionProperties",
    "SASCredential",
    "SASCredentialDto",
    "SamplingAlgorithm",
    "SasDatastoreCredentials",
    "SasDatastoreSecrets",
    "ScaleSettings",
    "ScaleSettingsInformation",
    "Schedule",
    "ScheduleActionBase",
    "ScheduleBase",
    "ScheduleProperties",
    "ScheduleResourceArmPaginatedResult",
    "ScriptReference",
    "ScriptsToExecute",
    "Seasonality",
    "ServerlessComputeSettings",
    "ServiceManagedResourcesSettings",
    "ServicePrincipalDatastoreCredentials",
    "ServicePrincipalDatastoreSecrets",
    "ServiceTagDestination",
    "ServiceTagOutboundRule",
    "SetupScripts",
    "SharedPrivateLinkResource",
    "Sku",
    "SkuCapacity",
    "SkuResource",
    "SkuResourceArmPaginatedResult",
    "SkuSetting",
    "SslConfiguration",
    "StackEnsembleSettings",
    "StaticInputData",
    "StorageAccountDetails",
    "SweepJob",
    "SweepJobLimits",
    "SynapseSpark",
    "SynapseSparkProperties",
    "SystemCreatedAcrAccount",
    "SystemCreatedStorageAccount",
    "SystemData",
    "SystemService",
    "TableVertical",
    "TableVerticalFeaturizationSettings",
    "TableVerticalLimitSettings",
    "TargetLags",
    "TargetRollingWindowSize",
    "TargetUtilizationScaleSettings",
    "TensorFlow",
    "TextClassification",
    "TextClassificationMultilabel",
    "TextNer",
    "TmpfsOptions",
    "TopNFeaturesByAttribution",
    "TrackedResource",
    "TrainingSettings",
    "TrialComponent",
    "TriggerBase",
    "TritonModelJobInput",
    "TritonModelJobOutput",
    "TruncationSelectionPolicy",
    "UpdateWorkspaceQuotas",
    "UpdateWorkspaceQuotasResult",
    "UriFileDataVersion",
    "UriFileJobInput",
    "UriFileJobOutput",
    "UriFolderDataVersion",
    "UriFolderJobInput",
    "UriFolderJobOutput",
    "Usage",
    "UsageName",
    "UserAccountCredentials",
    "UserAssignedIdentity",
    "UserCreatedAcrAccount",
    "UserCreatedStorageAccount",
    "UserIdentity",
    "UsernamePasswordAuthTypeWorkspaceConnectionProperties",
    "VirtualMachine",
    "VirtualMachineImage",
    "VirtualMachineSchema",
    "VirtualMachineSchemaProperties",
    "VirtualMachineSecrets",
    "VirtualMachineSecretsSchema",
    "VirtualMachineSize",
    "VirtualMachineSizeListResult",
    "VirtualMachineSshCredentials",
    "VolumeDefinition",
    "VolumeOptions",
    "Webhook",
    "Workspace",
    "WorkspaceConnectionManagedIdentity",
    "WorkspaceConnectionPersonalAccessToken",
    "WorkspaceConnectionPropertiesV2",
    "WorkspaceConnectionPropertiesV2BasicResource",
    "WorkspaceConnectionPropertiesV2BasicResourceArmPaginatedResult",
    "WorkspaceConnectionSharedAccessSignature",
    "WorkspaceConnectionUsernamePassword",
    "WorkspaceListResult",
    "WorkspaceUpdateParameters",
    "ActionType",
    "AllocationState",
    "ApplicationSharingPolicy",
    "AssetProvisioningState",
    "AutoRebuildSetting",
    "Autosave",
    "BatchLoggingLevel",
    "BatchOutputAction",
    "BillingCurrency",
    "BlockedTransformers",
    "Caching",
    "CategoricalDataDriftMetric",
    "CategoricalDataQualityMetric",
    "CategoricalPredictionDriftMetric",
    "ClassificationModels",
    "ClassificationMultilabelPrimaryMetrics",
    "ClassificationPrimaryMetrics",
    "ClusterPurpose",
    "ComputeInstanceAuthorizationType",
    "ComputeInstanceState",
    "ComputePowerAction",
    "ComputeRecurrenceFrequency",
    "ComputeTriggerType",
    "ComputeType",
    "ComputeWeekDay",
    "ConnectionAuthType",
    "ConnectionCategory",
    "ContainerType",
    "CreatedByType",
    "CredentialsType",
    "DataAvailabilityStatus",
    "DataReferenceCredentialType",
    "DataType",
    "DatastoreType",
    "DeploymentProvisioningState",
    "DiagnoseResultLevel",
    "DistributionType",
    "EarlyTerminationPolicyType",
    "EgressPublicNetworkAccessType",
    "EmailNotificationEnableType",
    "EncryptionStatus",
    "EndpointAuthMode",
    "EndpointComputeType",
    "EndpointProvisioningState",
    "EndpointServiceConnectionStatus",
    "EnvironmentType",
    "EnvironmentVariableType",
    "FeatureAttributionMetric",
    "FeatureDataType",
    "FeatureImportanceMode",
    "FeatureLags",
    "FeaturizationMode",
    "ForecastHorizonMode",
    "ForecastingModels",
    "ForecastingPrimaryMetrics",
    "Goal",
    "IdentityConfigurationType",
    "ImageType",
    "InputDeliveryMode",
    "InstanceSegmentationPrimaryMetrics",
    "IsolationMode",
    "JobInputType",
    "JobLimitsType",
    "JobOutputType",
    "JobStatus",
    "JobTier",
    "JobType",
    "KeyType",
    "LearningRateScheduler",
    "ListViewType",
    "LoadBalancerType",
    "LogVerbosity",
    "ManagedNetworkStatus",
    "ManagedServiceIdentityType",
    "MaterializationStoreType",
    "ModelSize",
    "ModelTaskType",
    "MonitorComputeIdentityType",
    "MonitorComputeType",
    "MonitoringFeatureDataType",
    "MonitoringFeatureFilterType",
    "MonitoringInputDataType",
    "MonitoringNotificationType",
    "MonitoringSignalType",
    "MountAction",
    "MountState",
    "NCrossValidationsMode",
    "Network",
    "NodeState",
    "NodesValueType",
    "NumericalDataDriftMetric",
    "NumericalDataQualityMetric",
    "NumericalPredictionDriftMetric",
    "ObjectDetectionPrimaryMetrics",
    "OperatingSystemType",
    "OperationName",
    "OperationStatus",
    "OperationTrigger",
    "OrderString",
    "Origin",
    "OsType",
    "OutputDeliveryMode",
    "PendingUploadCredentialType",
    "PendingUploadType",
    "PrivateEndpointConnectionProvisioningState",
    "PrivateEndpointServiceConnectionStatus",
    "Protocol",
    "ProvisioningState",
    "ProvisioningStatus",
    "PublicNetworkAccess",
    "PublicNetworkAccessType",
    "QuotaUnit",
    "RandomSamplingAlgorithmRule",
    "RecurrenceFrequency",
    "ReferenceType",
    "RegressionModels",
    "RegressionPrimaryMetrics",
    "RemoteLoginPortPublicAccess",
    "RuleAction",
    "RuleCategory",
    "RuleStatus",
    "RuleType",
    "SamplingAlgorithmType",
    "ScaleType",
    "ScheduleActionType",
    "ScheduleListViewType",
    "ScheduleProvisioningState",
    "ScheduleProvisioningStatus",
    "ScheduleStatus",
    "SeasonalityMode",
    "SecretsType",
    "ServiceDataAccessAuthIdentity",
    "ShortSeriesHandlingConfiguration",
    "SkuScaleType",
    "SkuTier",
    "SourceType",
    "SshPublicAccess",
    "SslConfigStatus",
    "StackMetaLearnerType",
    "Status",
    "StochasticOptimizer",
    "StorageAccountType",
    "TargetAggregationFunction",
    "TargetLagsMode",
    "TargetRollingWindowSizeMode",
    "TaskType",
    "TriggerType",
    "UnderlyingResourceAction",
    "UnitOfMeasure",
    "UsageUnit",
    "UseStl",
    "VMPriceOSType",
    "VMTier",
    "ValidationMetricType",
    "ValueFormat",
    "VmPriority",
    "VolumeDefinitionType",
    "WebhookType",
    "WeekDay",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
