# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import Artifact
    from ._models_py3 import Asset
    from ._models_py3 import AssetDto
    from ._models_py3 import AssetPaginatedResult
    from ._models_py3 import BatchGetResolvedUrisDto
    from ._models_py3 import BatchModelPathResponseDto
    from ._models_py3 import BlobReference
    from ._models_py3 import BlobReferenceForConsumptionDto
    from ._models_py3 import ContainerResourceRequirements
    from ._models_py3 import CreateUnregisteredInputModelDto
    from ._models_py3 import CreateUnregisteredOutputModelDto
    from ._models_py3 import CreatedBy
    from ._models_py3 import CreationContext
    from ._models_py3 import DataItem
    from ._models_py3 import DataReferenceCredentialDto
    from ._models_py3 import DataReferences
    from ._models_py3 import DataReferencesForConsumptionDto
    from ._models_py3 import DatasetReference
    from ._models_py3 import DependencyMapDto
    from ._models_py3 import DependencyMapItemDto
    from ._models_py3 import DependentAsset
    from ._models_py3 import DependentEntitiesDto
    from ._models_py3 import ErrorResponse
    from ._models_py3 import ExtensiveModel
    from ._models_py3 import FeedIndexEntityDto
    from ._models_py3 import FeedIndexEntityRequestDto
    from ._models_py3 import ImageReference
    from ._models_py3 import ImageReferenceForConsumptionDto
    from ._models_py3 import IndexAnnotations
    from ._models_py3 import IndexEntity
    from ._models_py3 import IndexProperties
    from ._models_py3 import InnerErrorDetails
    from ._models_py3 import IntellectualPropertyPublisherInformation
    from ._models_py3 import ListModelsRequest
    from ._models_py3 import Model
    from ._models_py3 import ModelBatchDto
    from ._models_py3 import ModelBatchResponseDto
    from ._models_py3 import ModelContainerRequest
    from ._models_py3 import ModelDeploymentSettings
    from ._models_py3 import ModelListModelsRequestPagedResponse
    from ._models_py3 import ModelPagedResponse
    from ._models_py3 import ModelPathResponseDto
    from ._models_py3 import ModelSchema
    from ._models_py3 import ModelSettingsIdentifiers
    from ._models_py3 import Operation
    from ._models_py3 import ProviderFeedEntityRequestDto
    from ._models_py3 import Relationship
    from ._models_py3 import ServiceResponseBase
    from ._models_py3 import User
except (SyntaxError, ImportError):
    from ._models import Artifact  # type: ignore
    from ._models import Asset  # type: ignore
    from ._models import AssetDto  # type: ignore
    from ._models import AssetPaginatedResult  # type: ignore
    from ._models import BatchGetResolvedUrisDto  # type: ignore
    from ._models import BatchModelPathResponseDto  # type: ignore
    from ._models import BlobReference  # type: ignore
    from ._models import BlobReferenceForConsumptionDto  # type: ignore
    from ._models import ContainerResourceRequirements  # type: ignore
    from ._models import CreateUnregisteredInputModelDto  # type: ignore
    from ._models import CreateUnregisteredOutputModelDto  # type: ignore
    from ._models import CreatedBy  # type: ignore
    from ._models import CreationContext  # type: ignore
    from ._models import DataItem  # type: ignore
    from ._models import DataReferenceCredentialDto  # type: ignore
    from ._models import DataReferences  # type: ignore
    from ._models import DataReferencesForConsumptionDto  # type: ignore
    from ._models import DatasetReference  # type: ignore
    from ._models import DependencyMapDto  # type: ignore
    from ._models import DependencyMapItemDto  # type: ignore
    from ._models import DependentAsset  # type: ignore
    from ._models import DependentEntitiesDto  # type: ignore
    from ._models import ErrorResponse  # type: ignore
    from ._models import ExtensiveModel  # type: ignore
    from ._models import FeedIndexEntityDto  # type: ignore
    from ._models import FeedIndexEntityRequestDto  # type: ignore
    from ._models import ImageReference  # type: ignore
    from ._models import ImageReferenceForConsumptionDto  # type: ignore
    from ._models import IndexAnnotations  # type: ignore
    from ._models import IndexEntity  # type: ignore
    from ._models import IndexProperties  # type: ignore
    from ._models import InnerErrorDetails  # type: ignore
    from ._models import IntellectualPropertyPublisherInformation  # type: ignore
    from ._models import ListModelsRequest  # type: ignore
    from ._models import Model  # type: ignore
    from ._models import ModelBatchDto  # type: ignore
    from ._models import ModelBatchResponseDto  # type: ignore
    from ._models import ModelContainerRequest  # type: ignore
    from ._models import ModelDeploymentSettings  # type: ignore
    from ._models import ModelListModelsRequestPagedResponse  # type: ignore
    from ._models import ModelPagedResponse  # type: ignore
    from ._models import ModelPathResponseDto  # type: ignore
    from ._models import ModelSchema  # type: ignore
    from ._models import ModelSettingsIdentifiers  # type: ignore
    from ._models import Operation  # type: ignore
    from ._models import ProviderFeedEntityRequestDto  # type: ignore
    from ._models import Relationship  # type: ignore
    from ._models import ServiceResponseBase  # type: ignore
    from ._models import User  # type: ignore

from ._azure_machine_learning_workspaces_enums import (
    ComputeEnvironmentType,
    DeploymentType,
    EntityKind,
    ListViewType,
    ModelFormatEnum,
    ModelSchemaDataType,
    OrderString,
    WebServiceState,
)

__all__ = [
    'Artifact',
    'Asset',
    'AssetDto',
    'AssetPaginatedResult',
    'BatchGetResolvedUrisDto',
    'BatchModelPathResponseDto',
    'BlobReference',
    'BlobReferenceForConsumptionDto',
    'ContainerResourceRequirements',
    'CreateUnregisteredInputModelDto',
    'CreateUnregisteredOutputModelDto',
    'CreatedBy',
    'CreationContext',
    'DataItem',
    'DataReferenceCredentialDto',
    'DataReferences',
    'DataReferencesForConsumptionDto',
    'DatasetReference',
    'DependencyMapDto',
    'DependencyMapItemDto',
    'DependentAsset',
    'DependentEntitiesDto',
    'ErrorResponse',
    'ExtensiveModel',
    'FeedIndexEntityDto',
    'FeedIndexEntityRequestDto',
    'ImageReference',
    'ImageReferenceForConsumptionDto',
    'IndexAnnotations',
    'IndexEntity',
    'IndexProperties',
    'InnerErrorDetails',
    'IntellectualPropertyPublisherInformation',
    'ListModelsRequest',
    'Model',
    'ModelBatchDto',
    'ModelBatchResponseDto',
    'ModelContainerRequest',
    'ModelDeploymentSettings',
    'ModelListModelsRequestPagedResponse',
    'ModelPagedResponse',
    'ModelPathResponseDto',
    'ModelSchema',
    'ModelSettingsIdentifiers',
    'Operation',
    'ProviderFeedEntityRequestDto',
    'Relationship',
    'ServiceResponseBase',
    'User',
    'ComputeEnvironmentType',
    'DeploymentType',
    'EntityKind',
    'ListViewType',
    'ModelFormatEnum',
    'ModelSchemaDataType',
    'OrderString',
    'WebServiceState',
]
