# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._operations import Operations
from ._deployments_operations import DeploymentsOperations
from ._providers_operations import ProvidersOperations
from ._resources_operations import ResourcesOperations
from ._resource_groups_operations import ResourceGroupsOperations
from ._tags_operations import TagsOperations
from ._deployment_operations_operations import DeploymentOperationsOperations

__all__ = [
    "Operations",
    "DeploymentsOperations",
    "ProvidersOperations",
    "ResourcesOperations",
    "ResourceGroupsOperations",
    "TagsOperations",
    "DeploymentOperationsOperations",
]
