# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------


class DataGenerationTaskType:
    NLI = "NLI"
    NLU_QA = "NLU_QA"
    CONVERSATION = "CONVERSATION"
    MATH = "MATH"
    SUMMARIZATION = "SUMMARIZATION"


class DataGenerationType:
    LABEL_GENERATION = "label_generation"
    DATA_GENERATION = "data_generation"
