# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------


class SnakeCaseFineTuningTaskTypes:
    CHAT_COMPLETION = "chat_completion"
    TEXT_COMPLETION = "text_completion"
    TEXT_CLASSIFICATION = "text_classification"
    QUESTION_ANSWERING = "question_answering"
    TEXT_SUMMARIZATION = "text_summarization"
    TOKEN_CLASSIFICATION = "token_classification"
    TEXT_TRANSLATION = "text_translation"
    IMAGE_CLASSIFICATION = "image_classification"
    IMAGE_INSTANCE_SEGMENTATION = "image_instance_segmentation"
    IMAGE_OBJECT_DETECTION = "image_object_detection"
    VIDEO_MULTI_OBJECT_TRACKING = "video_multi_object_tracking"
